mod sensors;
mod netspeed;
mod comm;
mod config;
mod external;

use comm::{Channel, Message};
use config::Config;
use std::collections::HashMap;
use std::sync::Arc;
use std::sync::mpsc;
use std::thread;

fn main() {
    let cfg = config::load("./panel.toml");
    let cfg = Arc::new(cfg);

    let (tx, rx) = mpsc::channel::<Message>();
    make_thread(&tx, &cfg, external::external);
    make_thread(&tx, &cfg, sensors::sensors);
    make_thread(&tx, &cfg, netspeed::netspeed);

    let mut data = HashMap::new();

    loop {
        let msg = rx.recv().unwrap();
        let empty = "".to_string();
        match msg {
            Message { kind, text } => data.insert(kind, text),
        };
        println!("{}%{{r}}{} | {}",
            data.get("title").unwrap_or(&empty),
            data.get("netspeed").unwrap_or(&empty),
            data.get("sensors").unwrap_or(&empty)
        );
    }
}

fn make_thread(tx: &Channel, cfg: &Arc<Config>, func: fn(&Channel, &Config) -> ()) {
    let thread_tx = tx.clone();
    let thread_cfg = cfg.clone();
    thread::spawn(move || {
        func(&thread_tx, &thread_cfg);
    });
}


